文本规则解析框架

####框架要点
1. 规则的基本构成  
规则一般包含两个部分，条件和动作。如下图所示
条件用来实现某种判断，如满足‘未成年人犯罪’，只需要满足被告人年龄小于等于18岁这个条件。
在案例解析任务中，条件的主要形式是用于审理程序、案由的判断以及段落句子的限定。
动作是根据条件执行某种逻辑操作，如文本匹配、内容提取、内容替换、执行脚本等。
2. 规则引用  
通常一个完整的语义包含主语、谓语、宾语。  
由于语言表达是复杂多样的，同样的主语、宾语，可以有不一样的表达方式，谓语的表达就更加多样化。
我们可以将一个完整的句子分成多个子的部分，每个部分独自扩展任意多种表达，然后再将各个部分重新组成完整的句子。
这需要实现一种表达式的引用，正则表达式本身没有提供多个独立表达式之间引用的语法，
我们在正则表达式的基础上做了一个小小的修改，对每一个规则指定一个唯一的名称，
将规则名称使用一对尖括号括起来表示对该规则的引用，  
如 '冒充公检法工作人员' 可以编写如下的规则表达式：
冒充公检法工作人员：<冒充><any><公检法工作人员>  
冒充：冒充|冒用|假借  
公检法工作人员：公安|民警|检察官|检察院|法院|法官|邮政|公检法|警察  
其中 "冒充"和"公检法工作人员"分别是两个独立的简单规则。  
规则引用的好处是显而易见的，通过这种方式可以增加规则表达式的可读性，同时方便语义片段的复用。
3. 规则上下文
规则的上下文分为外部上下文和内部上下午。
规则的条件判断需要从外部上下文中获取有用的信息，
规则的执行结果也要输出到外部环境。
规则的内部上下文用于临时存储规则从外部获取的断言、规则执行过程提取的信息，
并最终根据规则的执行情况决定是否合并到外部上下文。
我们约定使用context表示规则的外部环境上下文，_context表示内部上下文。
规则的执行过程就是内部上下文和外部上下文交换信息的过程。
4. 复合规则
复合规则用于构建复杂的规则，如下面几种形式的复合规则表达式：
表达同样的意思但是有多种不同的句式
A|B|C
(A1|A2)|(B1|B2)|(C1|C2)

必须同时满足多个条件
A&B&C
A|B|(C1&C2&C3)

利用复合规则的中断机制可以实现简单的规则执行顺序控制，提高执行效率

A|B|C  表示按照 A、B、C的顺序执行，一旦规则执行成功则中断
A&B&C  表示按照 A、B、C的顺序执行，一旦规则执行失败则中断


5. 系统规则
为一些非常通用的和使用频繁的表达编写规则是非常有用的。如表达匹配任意字符的规则、表达排除指定字符的规则、表达任意数值的规则、表达时间的规则等。
我们把这些规则作为系统规则发布，可以降低具体规则的编写难度。
规则解析图
规则解析图是由条件结点和规则结点构成的有向图。边的指向表示规则的执行方向。
规则图的执行本质上是规则图的遍历问题，
前置节点可以作为其邻接节点的前置条件，
所以需要控制某个结点规则在其所有前置节点之后执行。
很容易我们想到使用有向图的BFS算法实现规则图的遍历，
但是对于下面这种情况，BFS算法无法保证F节点在E节点之后执行。

分析原因是BFS算法能够保证按照一种广度优先的策略来遍历图上的节点，
但是不能保证这个遍历节点一定在其所有的前置节点之后。
我们希望总体按照广度优先进行遍历，但是同时希望前置节点一定先执行，
所以我们在BSF的基础上做了一些调整。
通过考虑节点的入度来控制，
我们在BFS算法执行过程中增加如下判断：
如果当前遍历的节点的入度大于1，将其入度减1；否则标记为已遍历
继续执行BFS算法直到所有的结点全部遍历完成
一轮遍历的过程中，当F节点的入度没有小于等于1， 
就算遍历到了也不会执行，
这样就能保证后置节点一定在其全部前置节点之后执	行，
即F结点在 B结点、E结点、D结点之后执行。

#### 安装依赖
- memcache  
安装说明：（参考文档https://www.cnblogs.com/lijunjiang2015/p/7995131.html）
wget http://memcached.org/latest/memcached-1.5.20.tar.gz
tar -zxvf memcached-1.x.x.tar.gz
cd memcached-1.x.x
./configure && make && make test && sudo make install # make test可以省略

- pymemcache  
使用说明：（参考文档https://pymemcache.readthedocs.io/en/latest/getting_started.html#basic-usage）

#### 源码介绍 （ruleparse/src/）
- ruleparse 框架的核心代码，文本匹配和信息抽取的基本框架
- phone 手机号码库
- timeNlp 时间抽取的库